from pagemenu.pagemenus import PageMenu
from pagemenu.items import MostLikedItem, MostRecentItem, ThisWeekItem, ThisMonthItem

class ContentPageMenu(PageMenu):
    def __init__(self, queryset, request):
        self.items = [
            MostRecentItem(
                request=request,
                title="Most Recent",
                get={'name': 'by', 'value': 'most-recent'},
                field_name='created',
                default=True,
            ),
            MostLikedItem(
                request=request,
                title="Most Liked",
                get={'name': 'by', 'value': 'most-liked'},
                default=False,
            ),
            ThisWeekItem(
                request=request,
                title="This Week",
                get={'name': 'for', 'value': 'this-week'},
                field_name='created',
                default=False,
            ),
            ThisMonthItem(
                request=request,
                title="This Month",
                get={'name': 'for', 'value': 'this-month'},
                field_name='created',
                default=False,
            )
        ]
        super(ContentPageMenu, self).__init__(queryset, request)



from django.conf import settings
from django.db import models

class PermittedManager(models.Manager):
    def get_query_set(self):
        # get base queryset and exclude based on state
        queryset = super(PermittedManager, self).get_query_set().exclude(state='unpublished')

        # exclude objects in staging state if not in staging mode (settings.STAGING = False)
        if not getattr(settings, 'STAGING', False):
            queryset = queryset.exclude(state='staging')

        # filter objects for current site 
        queryset = queryset.filter(sites__id__exact=settings.SITE_ID)
        return queryset



from copy import deepcopy

from django.db.models.fields import FieldDoesNotExist
from django import forms
from django.contrib import admin
from django.contrib.sites.models import Site
from django.contrib.auth.models import User

from panya.models import ModelBase
from publisher.models import Publisher
from photologue.admin import ImageOverrideInline

class ModelBaseAdminForm(forms.ModelForm):
    sites = forms.ModelMultipleChoiceField(
        queryset=Site.objects.all(), 
        help_text='Makes item eligible to be published on selected sites.',
        required=False, 
        widget=forms.CheckboxSelectMultiple()
    )
    publishers = forms.ModelMultipleChoiceField(
        queryset=Publisher.objects.all(), 
        help_text='Makes item eligible to be published on selected platform.',
        required=False, 
        widget=forms.CheckboxSelectMultiple())
    class Meta:
        model = ModelBase

class ModelBaseAdmin(admin.ModelAdmin):
    form = ModelBaseAdminForm
    inlines = [ImageOverrideInline,]
    list_display = ('title', 'state', 'admin_thumbnail', 'owner', 'created')
    list_filter = ('state', 'created')
    search_fields = ('title', 'description', 'state', 'created')
    fieldsets = (
        (None, {'fields': ('title', 'description', )}),
        ('Publishing', {'fields': ('state', 'publish_on', 'retract_on', 'sites', 'publishers'),
                    'classes': ('collapse',),
        }),
        ('Meta', {'fields': ('categories', 'tags', 'created', 'owner'),
                    'classes': ('collapse',),
        }),
        ('Image', {'fields': ('image', 'crop_from', 'effect'),
                    'classes': ('collapse',),
        }),
        ('Commenting', {'fields': ('comments_enabled', 'anonymous_comments', 'comments_closed'),
                    'classes': ('collapse',),
        }),
        ('Liking', {'fields': ('likes_enabled', 'anonymous_likes', 'likes_closed'),
                    'classes': ('collapse',),
        }),
    )
    
    def __init__(self, model, admin_site):
        super(ModelBaseAdmin, self).__init__(model, admin_site)
        fieldsets = deepcopy(ModelBaseAdmin.fieldsets)

        set_fields = []
        for fieldset in self.fieldsets:
            set_fields += fieldset[1]['fields']

        new_fields = []
        for name in model._meta.get_all_field_names():
            try:
                field = model._meta.get_field(name)
            except FieldDoesNotExist:
                continue

            # don't include custom through relations
            # custom if it has more than 3 fields
            try:
                if len(field.rel.through._meta.get_all_field_names()) > 3:
                    continue
            except AttributeError:
                pass

            if field.editable and field.formfield():
                if name not in set_fields and name not in ['id',]:
                    new_fields += [name,]
              
        for fieldset in fieldsets:
            if fieldset[0] == None:
                fieldset[1]['fields'] += tuple(new_fields)

        self.fieldsets = fieldsets

    
    def save_model(self, request, obj, form, change):
        if not obj.owner:
            obj.owner = request.user
      
        return super(ModelBaseAdmin, self).save_model(request, obj, form, change)



from django.core.urlresolvers import reverse
from django.shortcuts import get_object_or_404

from category.models import Category
from panya.generic.views import GenericObjectDetail, GenericObjectList
from panya.models import ModelBase
from panya.view_modifiers import DefaultViewModifier

class CategoryURL(object):
    def __init__(self, category):
        self.category = category

    def __call__(self, obj=None):
        if obj:
            return reverse('content_category_object_detail', kwargs={'category_slug': self.category.slug, 'slug': obj.slug})
        else:
            return self

class CategoryObjectList(GenericObjectList):
    def get_queryset(self, *args, **kwargs):
        return ModelBase.permitted.filter(categories=self.category)
   
    def get_view_modifier(self, request, *args, **kwargs):
        return DefaultViewModifier(request)

    def get_paginate_by(self, *args, **kwargs):
        return 7

    def get_url_callable(self, *args, **kwargs):
        return CategoryURL(category=self.category)
   
    def get_extra_context(self, *args, **kwargs):
        return {'title': self.category.title }
   
    def __call__(self, request, category_slug, *args, **kwargs):
        self.category = get_object_or_404(Category, slug__iexact=category_slug)
        return super(CategoryObjectList, self).__call__(request, *args, **kwargs)

category_object_list = CategoryObjectList()

class CategoryObjectDetail(GenericObjectDetail):
    def get_queryset(self, *args, **kwargs):
        return ModelBase.permitted.filter(categories=self.category)
    
    def get_view_modifier(self, request, *args, **kwargs):
        return DefaultViewModifier(request, base_url=reverse('content_category_object_list', kwargs={'category_slug': 'news-updates'}), ignore_defaults=True)
    
    def get_extra_context(self, *args, **kwargs):
        return {'title': self.category.title }
    
    def __call__(self, request, category_slug, *args, **kwargs):
        self.category = get_object_or_404(Category, slug__iexact=category_slug)
        return super(CategoryObjectDetail, self).__call__(request, *args, **kwargs)

category_object_detail = CategoryObjectDetail()

class ObjectPeek(GenericObjectDetail):
    def get_queryset(self, *args, **kwargs):
        return ModelBase.permitted

    def get_template_name(self, *args, **kwargs):
        return 'panya/modelbase_peek.html'
    
object_peek = ObjectPeek()



import random
import unittest
from datetime import datetime, timedelta

from django import template
from django.conf import settings
from django.contrib.admin.sites import AdminSite
from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ViewDoesNotExist
from django.core.urlresolvers import reverse
from django.db import models
from django.template import Template
from django.template.defaultfilters import slugify
from django.test import TestCase
from django.test.client import Client

from panya.admin import ModelBaseAdmin
from panya.models import ModelBase
from panya.utils.tests import RequestFactory
        
from photologue.models import PhotoSize
from secretballot.models import Vote
        
       
class DummyRelationalModel1(models.Model):
    pass
models.register_models('panya', DummyRelationalModel1)
class DummyRelationalModel2(models.Model):
    pass
models.register_models('panya', DummyRelationalModel2)
class DummyModel(ModelBase):
    test_editable_field = models.CharField(max_length=32)
    test_non_editable_field = models.CharField(max_length=32, editable=False)
    test_foreign_field = models.ForeignKey('DummyRelationalModel1', blank=True, null=True,)
    test_many_field = models.ManyToManyField('DummyRelationalModel2')
    test_member = True
models.register_models('panya', DummyModel)
class TrunkModel(ModelBase):
    pass
models.register_models('panya', TrunkModel)
class BranchModel(TrunkModel):
    pass
models.register_models('panya', BranchModel)
class LeafModel(BranchModel):
    pass
models.register_models('panya', LeafModel)
class TestModel(ModelBase):
    pass
models.register_models('panya', TestModel)

class UtilsTestCase(unittest.TestCase):
    def test_generate_slug(self):
        # on save a slug should be set
        obj = ModelBase(title='utils test case title')
        obj.save()
        self.failIf(obj.slug=='')

        # slug should become sluggified version of title
        obj = ModelBase(title='utils test case title 1')
        obj.save()
        self.failUnless(obj.slug==slugify(obj.title))

        # no two items should have the same slug
        obj = ModelBase(title='utils test case title 1')
        obj.save()

        # in case an object title is updated, the slug should also be updated
        obj.title = "updated title"
        obj.save()
        self.failUnless(obj.slug==slugify(obj.title))
        
        # in case an object is updated, without the title being changed, the slug should remain unchanged
        orig_slug = obj.slug
        obj.save()
        self.failUnless(obj.slug==orig_slug)

        # make sure the slug is actually saved
        obj = ModelBase.objects.get(id=obj.id)
        self.failIf(obj.slug=='')

class ModelBaseTestCase(unittest.TestCase):
    def test_save(self):
        before_save = datetime.now()

        # created field should be set on save
        obj = ModelBase(title='title')
        obj.save()
        
        # created field should be set to current datetime on save
        after_save = datetime.now()
        self.failIf(obj.created > after_save or obj.created < before_save)

        # if a user supplies a created date use that instead of the current datetime
        test_datetime = datetime(2008, 10, 10, 12, 12)
        obj = ModelBase(title='title', created=test_datetime)
        obj.save()
        self.failIf(obj.created != test_datetime)

        # modified should be set to current datetime on each save
        before_save = datetime.now()
        obj = ModelBase(title='title')
        obj.save()
        after_save = datetime.now()
        self.failIf(obj.modified > after_save or obj.modified < before_save)

        # leaf class content type should be set on save
        obj = DummyModel(title='title')
        obj.save()
        self.failUnless(obj.content_type == ContentType.objects.get_for_model(DummyModel))
        
        # leaf class class name should be set on save
        self.failUnless(obj.class_name == DummyModel.__name__)

        # correct leaf class content type should be retained over base class' content type
        base = obj.modelbase_ptr
        base.save()
        self.failUnless(base.content_type == ContentType.objects.get_for_model(DummyModel))
       
        # correct leaf class class name should be retained over base class' class name
        self.failUnless(base.class_name == DummyModel.__name__)

    def test_as_leaf_class(self):
        obj = LeafModel(title='title')
        obj.save()

        # always return the leaf class, no matter where we are in the hierarchy
        self.failUnless(TrunkModel.objects.get(slug=obj.slug).as_leaf_class() == obj)
        self.failUnless(BranchModel.objects.get(slug=obj.slug).as_leaf_class() == obj)
        self.failUnless(LeafModel.objects.get(slug=obj.slug).as_leaf_class() == obj)

    def test_vote_total(self):
        # create object with some votes
        obj = ModelBase(title='title')
        obj.save()
        obj.add_vote("token1", 1)
        obj.add_vote("token2", -1)
        obj.add_vote("token3", 1)

        # vote_total should return an integer
        result = obj.vote_total
        self.failUnlessEqual(result.__class__, int)

        # vote total is calculated as total_upvotes - total_downvotes
        self.failUnlessEqual(result, 1)

    def test_is_permitted(self):
        # create website site item and set as current site
        web_site = Site(domain="web.address.com")
        web_site.save()
        settings.SITE_ID = web_site.id

        # create unpublished item
        unpublished_obj = ModelBase(title='title', state='unpublished')
        unpublished_obj.save()
        unpublished_obj.sites.add(web_site)
        unpublished_obj.save()
        
        # create published item
        published_obj = ModelBase(title='title', state='published')
        published_obj.save()
        published_obj.sites.add(web_site)
        published_obj.save()
        
        # create staging item
        staging_obj = ModelBase(title='title', state='staging')
        staging_obj.save()
        staging_obj.sites.add(web_site)
        staging_obj.save()
        
        # is_permitted should be False for unpublished objects
        self.failIf(unpublished_obj.is_permitted)

        # is_permitted should be True for published objects
        self.failUnless(published_obj.is_permitted)
        
        # is_permitted should be True for otherwise published objects in the staging state for instances that define settings.STAGING = True
        settings.STAGING = False
        self.failIf(staging_obj.is_permitted)
        settings.STAGING = True
        self.failUnless(staging_obj.is_permitted)
        
        # is_permitted should be True only if the object is published for the current site
        published_obj_web = ModelBase(state='published')
        published_obj_web.save()
        published_obj_web.sites.add(web_site)
        published_obj_web.save()
        self.failUnless(published_obj_web.is_permitted)
        
        # is_permitted should be False if the object is not published for the current site
        mobile_site = Site(domain="mobi.address.com")
        mobile_site.save()
        published_obj_mobile = ModelBase(state='published')
        published_obj_mobile.save()
        published_obj_mobile.sites.add(mobile_site)
        published_obj_mobile.save()
        self.failIf(published_obj_mobile.is_permitted)

    def test_can_vote(self):
        # create dummy request object
        request = type('Request', (object,), {})
        class User():
            def is_authenticated(self):
                return False
        request.user = User()
        request.secretballot_token = 'test_token'

        # return false when liking is closed
        obj = ModelBase(likes_enabled=True, likes_closed=True, anonymous_likes=True)
        obj.save()
        self.failIf(obj.can_vote(request))

        # return false when liking is disabled
        obj = ModelBase(likes_enabled=False, likes_closed=False, anonymous_likes=True)
        obj.save()
        self.failIf(obj.can_vote(request))
        
        # return false if anonymous and anonymous liking is disabled
        obj = ModelBase(likes_enabled=True, likes_closed=False, anonymous_likes=False)
        obj.save()
        self.failIf(obj.can_vote(request))
        
        # return true if anonymous and anonymous liking is enabled
        obj = ModelBase(likes_enabled=True, likes_closed=False, anonymous_likes=True)
        obj.save()
        self.failUnless(obj.can_vote(request))

        # return false if vote already exist
        content_type = ContentType.objects.get(app_label="panya", model="modelbase")
        Vote.objects.create(object_id=obj.id, token='test_token', content_type=content_type, vote=1)
        self.failIf(obj.can_vote(request))
    
    def test_can_comment(self):
        # create dummy request object
        request = type('Request', (object,), {})
        class User():
            def is_authenticated(self):
                return False
        request.user = User()
        request.secretballot_token = 'test_token'

        # return false when commenting is closed
        obj = ModelBase(comments_enabled=True, comments_closed=True, anonymous_comments=True)
        obj.save()
        self.failIf(obj.can_comment(request))
        
        # return false when commenting is disabled
        obj = ModelBase(comments_enabled=False, comments_closed=False, anonymous_comments=True)
        obj.save()
        self.failIf(obj.can_comment(request))
        
        
        # return false if anonymous and anonymous commenting is disabled
        obj = ModelBase(comments_enabled=True, comments_closed=False, anonymous_comments=False)
        obj.save()
        self.failIf(obj.can_comment(request))
        
        # return true if anonymous and anonymous commenting is enabled
        obj = ModelBase(comments_enabled=True, comments_closed=False, anonymous_comments=True)
        obj.save()
        self.failUnless(obj.can_comment(request))

class ModelBaseAdminTestCase(unittest.TestCase):
    def setUp(self):
        self.user, self.created = User.objects.get_or_create(username='test', email='test@test.com')
   
    def test_field_hookup(self):
        model_admin = ModelBaseAdmin(DummyModel, AdminSite())
        
        # field additions should be added to first fieldsets' fields
        self.failIf('test_editable_field' not in model_admin.fieldsets[0][1]['fields'])
        self.failIf('test_foreign_field' not in model_admin.fieldsets[0][1]['fields'])
        self.failIf('test_many_field' not in model_admin.fieldsets[0][1]['fields'])
        
        # non editable field additions should not be added to fieldsets
        self.failIf('test_non_editable_field' in model_admin.fieldsets[0][1]['fields'])

        # non field class members should not be added to fieldsets
        self.failIf('test_member' in model_admin.fieldsets[0][1]['fields'])

    def test_save_model(self):
        # setup mock objects
        admin_obj = ModelBaseAdmin(ModelBase, 1)
        request = RequestFactory()
        request.user = self.user

        # after admin save the object's owner should be the current user 
        obj = ModelBase()
        admin_obj.save_model(request, obj, admin_obj.form, 1)
        self.failUnless(obj.owner == self.user)
        obj.save()
       
        # TODO: if a different user is specified as owner set that user as owner

class PermittedManagerTestCase(unittest.TestCase):
    def setUp(self):
        # create website site item and set as current site
        self.web_site = Site(domain="web.address.com")
        self.web_site.save()
        settings.SITE_ID = self.web_site.id

    def test_get_query_set(self):
        # create unpublished item
        unpublished_obj = ModelBase(title='title', state='unpublished')
        unpublished_obj.save()
        unpublished_obj.sites.add(self.web_site)
        unpublished_obj.save()
        
        # create published item
        published_obj = ModelBase(title='title', state='published')
        published_obj.save()
        published_obj.sites.add(self.web_site)
        published_obj.save()
        
        # create staging item
        staging_obj = ModelBase(title='title', state='staging')
        staging_obj.save()
        staging_obj.sites.add(self.web_site)
        staging_obj.save()
        
        # unpublished objects should not be available in queryset
        queryset = ModelBase.permitted.all()
        self.failIf(unpublished_obj in queryset)

        # published objects should always be available in queryset
        self.failUnless(published_obj in queryset)
        
        # staging objects should only be available on instances that define settings.STAGING = True
        settings.STAGING = False
        queryset = ModelBase.permitted.all()
        self.failIf(staging_obj in queryset)
        settings.STAGING = True
        queryset = ModelBase.permitted.all()
        self.failUnless(staging_obj in queryset)
        
        # queryset should only contain items for the current site
        published_obj_web = ModelBase(state='published')
        published_obj_web.save()
        published_obj_web.sites.add(self.web_site)
        published_obj_web.save()
        queryset = ModelBase.permitted.all()
        self.failUnless(published_obj_web in queryset)
        
        # queryset should not contain items for other sites
        mobile_site = Site(domain="mobi.address.com")
        mobile_site.save()
        published_obj_mobile = ModelBase(state='published')
        published_obj_mobile.save()
        published_obj_mobile.sites.add(mobile_site)
        published_obj_mobile.save()
        queryset = ModelBase.permitted.all()
        self.failIf(published_obj_mobile in queryset)

    def test_content_type(self):
        obj = BranchModel(title='title', state='published')
        obj.save()
        obj.sites.add(self.web_site)
        obj.save()

        # queryset should return objects of the same type as the queried model
        queryset = BranchModel.permitted.all()
        self.failUnless(obj in queryset)
        queryset = ModelBase.permitted.all()
        self.failIf(obj in queryset)

class InclusionTagsTestCase(unittest.TestCase):
    def setUp(self):
        obj = TestModel(title='title', state='published')
        obj.save()
        self.context = template.Context({'object': obj})

    def test_render_tag(self):
        # load correct template for provided object and type
        t = Template("{% load panya_inclusion_tags %}{% render_object object 'block' %}")
        result = t.render(self.context)
        expected_result = u'Test string for testing purposes\n'
        self.failUnlessEqual(result, expected_result)

        # if template is not available for object, fall back to default content template
        obj = BranchModel(title='title', state='published')
        obj.save()
        self.context = template.Context({'object': obj})
        t = Template("{% load panya_inclusion_tags %}{% render_object object 'block' %}")
        result = t.render(self.context)
        self.failUnless(result)

        # return the empty string if no template can be found for the given type for either obj or content.
        t = Template("{% load panya_inclusion_tags %}{% render_object object 'foobar' %}")
        result = t.render(self.context)
        expected_result = u''
        self.failUnlessEqual(result, expected_result)

class TemplateTagsTestCase(unittest.TestCase):
    def setUp(self):
        def url_callable(obj):
            return 'Test URL method using object %s' % obj.__class__.__name__

        obj = TestModel(title='title', state='published')
        obj.save()
        self.context = template.Context({'object': obj, 'url_callable': url_callable})

    def test_smart_url(self):
        # return method call with result based on object provided
        t = Template("{% load panya_template_tags %}{% smart_url url_callable object %}")
        result = t.render(self.context)
        self.failUnlessEqual(result, 'Test URL method using object TestModel') 



from django.conf.urls.defaults import patterns, url

urlpatterns = patterns(
    'panya.views',
    url(r'^peek/(?P<slug>[\w-]+)/$', 'object_peek', name='object_peek'),
    url(r'^(?P<category_slug>[\w-]+)/list/$', 'category_object_list', name='content_category_object_list'),
    url(r'^(?P<category_slug>[\w-]+)/(?P<slug>[\w-]+)/$', 'category_object_detail', name='content_category_object_detail'),
)



import time
from datetime import datetime

from django import template
from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.db.models import signals
from django.template import Template

import secretballot
from panya.managers import PermittedManager
from panya.utils import generate_slug

from photologue.models import ImageModel
from secretballot.models import Vote

class ModelBase(ImageModel):
    objects = models.Manager()
    permitted = PermittedManager()
    
    state = models.CharField(
        max_length=32,
        choices=(
            ('unpublished', 'Unpublished'),
            ('published', 'Published'),
            ('staging', 'Staging'),
        ),
        default='unpublished',
        help_text="Set the item state. The 'Published' state makes the item visible to the public, 'Unpublished' retracts it and 'Staging' makes the item visible to staff users.",
        blank=True,
        null=True,
    )
    publish_on = models.DateTimeField(
        blank=True,
        null=True,
        help_text="Date and time on which to publish this item (state will change to 'published').",
    )
    retract_on = models.DateTimeField(
        blank=True,
        null=True,
        help_text="Date and time on which to retract this item (state will change to 'unpublished').",
    )
    slug = models.SlugField(
        editable=False,
        max_length=255,
        db_index=True,
        unique=True,
    )
    title = models.CharField(
        max_length=200, help_text='A short descriptive title.',
    )
    description = models.TextField(
        help_text='A short description. More verbose than the title but limited to one or two sentences.',
        blank=True,
        null=True,
    )
    created = models.DateTimeField(
        'Created Date & Time', 
        blank=True,
        help_text='Date and time on which this item was created. This is automatically set on creation, but can be changed subsequently.'
    )
    modified = models.DateTimeField(
        'Modified Date & Time', 
        editable=False,
        help_text='Date and time on which this item was last modified. This is automatically set each time the item is saved.'
    )
    owner = models.ForeignKey(
        User, 
        blank=True,
        null=True,
    )
    content_type = models.ForeignKey(
        ContentType, 
        editable=False, 
        null=True
    )
    class_name = models.CharField(
        max_length=32, 
        editable=False, 
        null=True
    )
    categories = models.ManyToManyField(
        'category.Category',
        blank=True,
        null=True,
        help_text='Categorizing this item.'
    )
    tags = models.ManyToManyField(
        'category.Tag',
        blank=True,
        null=True,
        help_text='Tag this item.'
    )
    sites = models.ManyToManyField(
        'sites.Site',
        blank=True,
        null=True,
        help_text='Makes item eligible to be published on selected sites.',
    )
    publishers = models.ManyToManyField(
        'publisher.Publisher',
        blank=True,
        null=True,
        help_text='Makes item eligible to be published on selected platform.',
    )
    comments_enabled = models.BooleanField(
        verbose_name="Commenting Enabled",
        help_text="Enable commenting for this item. Comments will not display when disabled.",
        default=True,
    )
    anonymous_comments = models.BooleanField(
        verbose_name="Anonymous Commenting Enabled",
        help_text="Enable anonymous commenting for this item.",
        default=True,
    )
    comments_closed = models.BooleanField(
        verbose_name="Commenting Closed",
        help_text="Close commenting for this item. Comments will still display, but users won't be able to add new comments.",
        default=False,
    )
    likes_enabled = models.BooleanField(
        verbose_name="Liking Enabled",
        help_text="Enable liking for this item. Likes will not display when disabled.",
        default=True,
    )
    anonymous_likes = models.BooleanField(
        verbose_name="Anonymous Liking Enabled",
        help_text="Enable anonymous liking for this item.",
        default=True,
    )
    likes_closed = models.BooleanField(
        verbose_name="Liking Closed",
        help_text="Close liking for this item. Likes will still display, but users won't be able to the item anymore.",
        default=False,
    )
   
    class Meta:
        ordering = ('-created',)
    
    def as_leaf_class(self):
        """
        Returns the leaf class no matter where the calling instance is in the inheritance hierarchy.
        Inspired by http://www.djangosnippets.org/snippets/1031/
        """
        try:
            return self.__getattribute__(self.class_name.lower())
        except AttributeError:
            content_type = self.content_type
            model = content_type.model_class()
            if(model == ModelBase):
                return self
            return model.objects.get(id=self.id)
        
    def save(self, *args, **kwargs):
        # set created time to now on initial save.
        if not self.id and not self.created:
            self.created = datetime.now()

        # set modified to now on each save 
        self.modified = datetime.now()
       
        # set leaf class content type
        if not self.content_type:
            self.content_type = ContentType.objects.get_for_model(self.__class__)
        
        # set leaf class class name
        if not self.class_name:
            self.class_name = self.__class__.__name__

        # set title as slug uniquely
        self.slug = generate_slug(self, self.title)

        super(ModelBase, self).save(*args, **kwargs)

    def __unicode__(self):
        return self.title

    @property
    def is_permitted(self):
        def for_site():
            if self.sites.filter(id__exact=settings.SITE_ID):
                return True
            else:
                return False

        if self.state == 'unpublished':
            return False
        elif self.state == 'published':
            return for_site()
        elif self.state == 'staging':
            if getattr(settings, 'STAGING', False):
                return for_site()
            
        return False

    @property
    def modelbase_obj(self):
        if self.__class__ == ModelBase:
            return self
        else:
            return self.modelbase_ptr
       
    def can_vote(self, request):
        """
        Determnines whether or not the current user can vote.
        Returns a bool as well as a string indicating the current vote status,
        with vote status being one of: 'closed', 'disabled', 'auth_required', 'can_vote', 'voted' 
        """
        modelbase_obj = self.modelbase_obj

        # can't vote if liking is closed
        if modelbase_obj.likes_closed:
            return False, 'closed'
        
        # can't vote if liking is disabled
        if not modelbase_obj.likes_enabled:
            return False, 'disabled'

        # anonymous users can't vote if anonymous likes are disabled
        if not request.user.is_authenticated() and not modelbase_obj.anonymous_likes:
            return False, 'auth_required'
          
        # return false if existing votes are found
        if Vote.objects.filter(object_id=modelbase_obj.id, token=request.secretballot_token).count() == 0:
            return True, 'can_vote'
        else:
            return False, 'voted'
    
    def can_comment(self, request):
        modelbase_obj = self.modelbase_obj
       
        # can't comment if commenting is closed
        if modelbase_obj.comments_closed:
            return False

        
        # can't comment if commenting is disabled
        if not modelbase_obj.comments_enabled:
            return False
        
        # anonymous users can't comment if anonymous comments are disabled
        if not request.user.is_authenticated() and not modelbase_obj.anonymous_comments:
            return False
        
        return True
            
    @property
    def vote_total(self):
        """
        Calculates vote total as total_upvotes - total_downvotes. We are adding a method here instead of relying on django-secretballot's addition since that doesn't work for subclasses.
        """
        modelbase_obj = self.modelbase_obj
        return modelbase_obj.votes.filter(vote=+1).count() - modelbase_obj.votes.filter(vote=-1).count() 
    
    @property
    def comment_count(self):
        """
        Counts total number of comments on ModelBase object.
        """
        modelbase_obj = self.modelbase_obj
        t = Template("{% load comments %}{% get_comment_count for object as count %}{{ count }}")
        return int(t.render(template.Context({'object': modelbase_obj})))

def set_managers(sender, **kwargs):
    """
    Make sure all classes have the appropriate managers 
    """
    cls = sender
   
    if issubclass(cls, ModelBase):
        cls.add_to_class('permitted', PermittedManager())

signals.class_prepared.connect(set_managers)

# enable voting for ModelBase, but specify a different total name 
# so ModelBase's vote_total method is not overwritten
secretballot.enable_voting_on(ModelBase, total_name="secretballot_added_vote_total")



from datetime import datetime

from django import template

register = template.Library()

@register.simple_tag
def smart_url(url_callable, obj):
    return url_callable(obj)

@register.tag
def smart_query_string(parser, token):
    """
    Outputs current GET query string with additions appended.
    Additions are provided in token pairs. 
    """
    args = token.split_contents()
    additions = args[1:]
   
    addition_pairs = []
    while additions:
        addition_pairs.append(additions[0:2])
        additions = additions[2:]

    return SmartQueryStringNode(addition_pairs)

class SmartQueryStringNode(template.Node):
    def __init__(self, addition_pairs):
        self.addition_pairs = []
        for key, value in addition_pairs:
            self.addition_pairs.append((template.Variable(key) if key else None, template.Variable(value) if value else None))

    def render(self, context):
        q = dict([(k, v) for k, v in context['request'].GET.items()])
        for key, value in self.addition_pairs:
            if key:
                key = key.resolve(context)
                if value:
                   value = value.resolve(context)
                   q[key] = value
                else:
                    q.pop(key, None)
            qs = '&'.join(['%s=%s' % (k, v) for k, v in q.items()])
        return '?' + qs if len(q) else ''

@register.tag
def humanize_time_diff(parser, token):
    try:
        tag_name, date_obj, suffix = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError, "%s requires exactly two argument" % tag_name
    
    return HumanizeTimeDifference(date_obj, suffix)

class HumanizeTimeDifference(template.Node):
    """
    Adapted from Django Snippet 412
    
    Returns a humanized string representing time difference
    between now() and the input timestamp.
    
    The output rounds up to days, hours, minutes, or seconds.
    4 days 5 hours returns '4 days'
    0 days 4 hours 3 minutes returns '4 hours', etc...
    """
    def __init__(self, date_obj, suffix):
        self.date_obj = template.Variable(date_obj)
        self.suffix = template.Variable(suffix)

    def render(self, context):
        date_obj = self.date_obj.resolve(context)
        suffix = self.suffix.resolve(context)
        
        if date_obj:
            time_difference = datetime.now() - date_obj
            days = time_difference.days
            hours = time_difference.seconds / 3600
            minutes = time_difference.seconds % 3600 / 60
            seconds = time_difference.seconds % 3600 % 60
           
            if days > 0:
                if days == 1: return "Yesterday"
                else: dt_str = "Days"
                return "%s %s %s" % (days, dt_str, suffix)
            elif hours > 0:
                if hours == 1: dt_str = "Hour"
                else: dt_str = "Hours"
                return "%s %s %s" % (hours, dt_str, suffix)
            elif minutes > 0:
                if minutes == 1: dt_str = "Minute"
                else: dt_str = "Minutes"
                return "%s %s %s" % (minutes, dt_str, suffix)
            elif seconds > 0:
                if seconds == 1: dt_str = "Second"
                else: dt_str = "Seconds"
                return "%s %s %s" % (seconds, dt_str, suffix)
            elif seconds == 0:
                return "Just Now"
        return ""



import copy
import traceback

from django.db.models import Q
from django.db.models.fields import related 
from django.contrib import messages
from django.shortcuts import render_to_response
from django.template import loader
from django.template import RequestContext
from django.utils.translation import ugettext
from django.views.generic import list_detail

class DefaultURL(object):
    def __call__(self, obj=None):
        if obj:
            try:
                return obj.get_absolute_url()
            except AttributeError:
                return ''
        else:
            return self
    
class GenericBase(object):
    def __init__(self, *args, **kwargs):
        self.params=kwargs
        self.params['view_modifier'] = None
        self.params['url_callable'] = None

    def __call__(self, request, *args, **kwargs):
        self.request = request
        view = copy.copy(self)
        
        # setup view params
        view.params = view._resolve_view_params(request, view.defaults, *args, **kwargs)
      
        # push the view through view modifier
        if view.params['extra_context'].has_key('view_modifier'):
            view_modifier = view.params['extra_context']['view_modifier']
            if view_modifier:
                if callable(view_modifier):
                    view_modifier = view_modifier(request=request, *args, **kwargs)
                    view.params['extra_context']['view_modifier'] = view_modifier
                view = view_modifier.modify(view)

        return view

    def get_url_callable(self, *args, **kwargs):
        return DefaultURL()

    def _resolve_view_params(self, request, defaults, *args, **kwargs):
        """
        Resolves view params with least ammount of resistance.
        Firstly check for params on urls passed args, then on class init args or members, 
        and lastly on class get methods .
        """
        params = copy.copy(defaults)
        params.update(self.params)
        params.update(kwargs)
        resolved_params = {}
        
        extra_context = {}
        for key in params:
            # grab from provided params. 
            value = params[key]
            
            # otherwise grab from existing params
            if value == None:
                value = self.params[key] if self.params.has_key(key) else None
            
            # otherwise grab from class method
            if value == None:
                value = getattr(self, 'get_%s' % key)(request, *args, **kwargs) if getattr(self, 'get_%s' % key, None) else None

            if key in defaults:
                resolved_params[key] = value
            else:
                extra_context[key] = value
        
        if extra_context:
            try:
                resolved_params['extra_context'].update(extra_context)
            except AttributeError:
                resolved_params['extra_context'] = extra_context

        return resolved_params
        
class GenericObjectList(GenericBase):
    defaults = {
        'queryset': None,
        'paginate_by': None, 
        'page':None,
        'allow_empty':True, 
        'template_name':None, 
        'template_loader':loader,
        'extra_context':None, 
        'context_processors':None, 
        'template_object_name':'object',
        'mimetype':None,
    }
    
    def __call__(self, request, *args, **kwargs):
        # generate our view via genericbase
        view = super(GenericObjectList, self).__call__(request, *args, **kwargs)
        
        # setup object_list params
        queryset=view.params['queryset']
        del view.params['queryset']

        # return object list generic view
        return list_detail.object_list(request, queryset=queryset, **view.params)
        
generic_object_list = GenericObjectList()
        
class GenericObjectFilterList(GenericObjectList):
    
    def __call__(self, request, *args, **kwargs):
        # generate our view via genericbase
        view = super(GenericObjectList, self).__call__(request, *args, **kwargs)

        # setup object_list params
        queryset=view.params['queryset']
        del view.params['queryset']
        
        # Filter
        for field in queryset.model._meta.fields:
            if field.name in view.params['extra_context'].keys():
                queryset = queryset.filter(Q(**{"%s__exact" % field.name : view.params['extra_context'][field.name]}))
        
        # return object list generic view
        return list_detail.object_list(request, queryset=queryset, **view.params)
        
generic_object_filter_list = GenericObjectFilterList()

class GenericObjectDetail(GenericBase):
    defaults = {
        'queryset': None,
        'object_id': None, 
        'slug': None,
        'slug_field':'slug', 
        'template_name_field':None,
        'template_name':None, 
        'template_loader':loader,
        'extra_context':None, 
        'context_processors':None, 
        'template_object_name':'object',
        'mimetype':None,
    }
    
    def __call__(self, request, *args, **kwargs):
        # generate our view via genericbase
        view = super(GenericObjectDetail, self).__call__(request, *args, **kwargs)
        
        # setup object_list params
        queryset=view.params['queryset']
        del view.params['queryset']

        # return object list generic view
        return list_detail.object_detail(request, queryset=queryset, **view.params)
        
generic_object_detail = GenericObjectDetail()

class GenericForm(GenericBase):
    defaults = {
        'form_class': None,
        'form_args': None,
        'initial': None,
        'extra_context': None, 
        'template_name': None,
        'success_message': None,
    }
    
    def handle_valid(self, form=None, *args, **kwargs):
        """
        Called after the form has validated.
        """
        # Take a chance and try save a subclass of a ModelForm.
        if hasattr(form, 'save'):
            form.save()
        # Also try and call handle_valid method of the form itself.
        if hasattr(form, 'handle_valid'):
            form.handle_valid(*args, **kwargs)
    
    def redirect(self, request, *args, **kwargs):
        """
        Redirect after successful form submission.
        Default behaviour is to not redirect and hence return the original view.
        """
        return None
    
    def __call__(self, request, *args, **kwargs):
        # generate our view via genericbase
        view = super(GenericForm, self).__call__(request, *args, **kwargs)

        self.form_class = view.params['form_class']
        self.form_args = view.params['form_args']
        self.template_name = view.params['template_name']
        self.success_message = view.params['success_message']

        if request.method == 'POST':
            form = self.form_class(data=request.POST, files=request.FILES, **self.form_args)
            if form.is_valid():
                self.handle_valid(form=form, request=request, *args, **kwargs)
                if self.success_message:
                    msg = ugettext(self.success_message)
                    messages.success(request, msg, fail_silently=True)
                redirect = self.redirect(request, *args, **kwargs)
                if redirect:
                    return redirect
        else:
            form = self.form_class(initial=self.get_initial(request=request, *args, **kwargs), **self.form_args)
    
        context = RequestContext(request, {})
        context.update({
            'form': form,
        })
        context.update(view.params['extra_context'])
        return render_to_response(self.template_name, context)
        
generic_form_view = GenericForm()



from panya.view_modifiers.items import CalEntryUpcomingItem, CalEntryThisWeekendItem, CalEntryNext7DaysItem, CalEntryThisMonthItem, IntegerFieldRangeItem, MostRecentItem, MostLikedItem, TagItem, ThisMonthItem, ThisWeekItem

class ViewModifier(object):
    def __init__(self, request, ignore_defaults=False, *args, **kwargs):
        self.request = request
        self.ignore_defaults = ignore_defaults

        self.groups = {}
        for item in self.items:
            group = item.group
            if group:
                if self.groups.has_key(group):
                    self.groups[group].append(item)
                else:
                    self.groups[group] = [item,]

        self.active_items = self.get_active_items()

    def get_active_items(self):
        active_items = []
        active_groups = []
        for item in self.items:
            if item.is_active(self.request):
                active_items.append(item)
                if item.group:
                    active_groups.append(item.group)

        # for each group without an active item fall back to defaults,
        # but only if defaults are not ignored(ignore_defaults=False)
        if not self.ignore_defaults:
            if item.group:
                for item in self.items:
                    if item.default and item.group not in active_groups:
                        active_items.append(item)
                        active_groups.append(item.group)
        
        # if we still don't have any active items fall back to defaults that don't have groups,
        # but only if defaults are not ignored(ignore_defaults=False)
        if not active_items and not self.ignore_defaults:
            for item in self.items:
                if not item.group and item.default:
                    active_items.append(item)

        return active_items

    def modify(self, view):
        for item in self.active_items:
            view = item.modify(view)

        return view
    
class DateFieldIntervalViewModifier(ViewModifier):
    def __init__(self, request, field_name, base_url=None, *args, **kwargs):
        self.items = [
            CalEntryUpcomingItem(
                request=request,
                title="Upcoming",
                get={'name': 'filter', 'value': 'recent'},
                field_name=field_name,
                base_url=base_url,
                default=True,
            ), 
            CalEntryThisWeekendItem(
                request=request,
                title="This Weekend",
                get={'name': 'filter', 'value': 'weekend'},
                field_name=field_name,
                base_url=base_url,
            ), 
            CalEntryNext7DaysItem(
                request=request,
                title="Next 7 Days",
                get={'name': 'filter', 'value': 'week'},
                field_name=field_name,
                base_url=base_url,
            ), 
            CalEntryThisMonthItem(
                request=request,
                title="This Month",
                get={'name': 'filter', 'value': 'month'},
                field_name=field_name,
                base_url=base_url,
            ), 
        ]
        super(DateFieldIntervalViewModifier, self).__init__(request, *args, **kwargs)

class DefaultViewModifier(ViewModifier):
    def __init__(self, request, base_url=None, *args, **kwargs):
        self.items = [
            MostRecentItem(
                request=request,
                title="Most Recent",
                get={'name': 'by', 'value': 'most-recent'},
                field_name='created',
                base_url=base_url,
                default=True,
            ),
            MostLikedItem(
                request=request,
                title="Most Liked",
                get={'name': 'by', 'value': 'most-liked'},
                base_url=base_url,
                default=False,
            ),
            ThisWeekItem(
                request=request,
                title="This Week",
                get={'name': 'for', 'value': 'this-week'},
                field_name='created',
                base_url=base_url,
                default=False,
            ),
            ThisMonthItem(
                request=request,
                title="This Month",
                get={'name': 'for', 'value': 'this-month'},
                field_name='created',
                base_url=base_url,
                default=False,
            )
        ]
        super(DefaultViewModifier, self).__init__(request, *args, **kwargs)

class IntegerFieldRangeViewModifier(ViewModifier):
    def __init__(self, request, field_name, count, interval, *args, **kwargs):
        self.items = []

        ranges = range(0, count, interval)
        i = 0
        for range_start in ranges:
            range_end = range_start + interval
            range_start = range_start + 1
            self.items.append(IntegerFieldRangeItem(
                request=request,
                title="%s-%s" % (range_start, range_end),
                get={'name': 'range', 'value': range_start},
                field_name=field_name,
                filter_range=(range_start, range_end),
                default=i==0,
            ))
            i += 1

        super(IntegerFieldRangeViewModifier, self).__init__(request, *args, **kwargs)

class CategoryTagViewModifier(ViewModifier):
    def __init__(self, request, category, *args, **kwargs):
        from category.models import Tag
        
        self.items = []
        tags = Tag.objects.filter(categories=category)

        for tag in tags:
            self.items.append(TagItem(
                request=request,
                title=tag.title,
                get={'name': 'tag', 'value': tag.slug},
                field_name='tag',
                tag=tag,
                default=False,
            ))
            
        super(CategoryTagViewModifier, self).__init__(request, *args, **kwargs)



from setuptools import setup, find_packages

setup(
    name='panya',
    version='0.0.8',
    description='Panya base app.',
    long_description = open('README.rst', 'r').read(),
    author='Praekelt Foundation',
    author_email='dev@praekelt.com',
    license='BSD',
    url='http://github.com/praekelt/panya',
    packages = find_packages(),
    dependency_links = [
        'http://dist.plone.org/thirdparty/',
        'https://github.com/praekelt/django-category/tarball/master#egg=django-category',
        'http://github.com/praekelt/django-photologue/tarball/2.4.praekelt#egg=django-photologue-2.4.praekelt',
        'https://github.com/praekelt/django-publisher/tarball/master#egg=django-publisher',
    ],
    install_requires = [
        'PIL',
        'django-category',
        'django-photologue==2.4.praekelt',
        'django-publisher',
        'django-secretballot',
    ],
    include_package_data=True,
    classifiers = [
        "Programming Language :: Python",
        "License :: OSI Approved :: BSD License",
        "Development Status :: 4 - Beta",
        "Operating System :: OS Independent",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
    zip_safe=False,
)



import inspect
import re
import types

from django.db.models import Q
from django.db.utils import IntegrityError
from django.template.defaultfilters import slugify

def generate_slug(obj, text, tail_number=0):
    from panya.models import ModelBase
    """
    Returns a new unique slug. Object must provide a SlugField called slug.
    URL friendly slugs are generated using django.template.defaultfilters' slugify.
    Numbers are added to the end of slugs for uniqueness.
    """
    # use django slugify filter to slugify
    slug = slugify(text)

    existing_slugs = [item.slug for item in ModelBase.objects.exclude(id=obj.id)]

    tail_number = 0
    new_slug = slug
    while new_slug in existing_slugs:
        new_slug = slugify("%s-%s" % (slug, tail_number))
        tail_number += 1

    return new_slug

def modify_class(original_class, modifier_class, override=True):
    """
    Adds class methods from modifier_class to original_class.
    If override is True existing methods in original_class are overriden by those provided by modifier_class.
    """
    # get members to add
    modifier_methods = inspect.getmembers(modifier_class, inspect.ismethod)
    
    # set methods
    for method_tuple in modifier_methods:
        name = method_tuple[0]
        method = method_tuple[1]
        if isinstance(method, types.UnboundMethodType):
            if hasattr(original_class, name) and not override:
                return None
            else:
                setattr(original_class, name, method.im_func)

def normalize_query(query_string,
                    findterms=re.compile(r'"([^"]+)"|(\S+)').findall,
                    normspace=re.compile(r'\s{2,}').sub):
    """
    Splits the query string in invidual keywords, getting rid of unecessary spaces
    and grouping quoted words together.
    Example:
      
    >>> normalize_query('  some random  words "with   quotes  " and   spaces')
    ['some', 'random', 'words', 'with quotes', 'and', 'spaces']
    """
    
    return [normspace(' ', (t[0] or t[1]).strip()) for t in findterms(query_string)] 

def get_query(query_string, search_fields):
    """
    Returns a query, that is a combination of Q objects. That combination
    aims to search keywords within a model by testing the given search fields.
    search_fields should be something like: [('title', 'iexact'), ('content', 'icontains'), ]
    """
    query = None # Query to search for every search term        
    terms = normalize_query(query_string)
    for term in terms:
        or_query = None # Query to search for a given term in each field
        for field in search_fields:
            q = Q(**{"%s__%s" % (field[0], field[1]): term})
            if or_query is None:
                or_query = q
            else:
                or_query = or_query | q
        if query is None:
            query = or_query
        else:
            query = query & or_query
    return query



from copy import copy

from django import template
from django.template import TemplateDoesNotExist
from django.contrib.contenttypes.models import ContentType
from django.template.loader import render_to_string

register = template.Library()

@register.inclusion_tag('panya/inclusion_tags/content_list_gizmo.html', takes_context=True)
def content_list_gizmo(context, object_list):
    context.update({'object_list': object_list})
    return context

@register.inclusion_tag('panya/inclusion_tags/modelbase_list.html', takes_context=True)
def modelbase_listing(context, object_list, type):
    context.update({
        'object_list': object_list,
        'type': type,
    })
    return context

@register.inclusion_tag('panya/inclusion_tags/object_comments.html', takes_context=True)
def object_comments(context, obj):
    context.update({'object': obj.modelbase_obj})
    return context

@register.inclusion_tag('panya/inclusion_tags/object_header.html', takes_context=True)
def object_header(context, obj):
    context.update({'object': obj})
    return context

@register.tag
def pager(parser, token):
    """
    Output pagination links.
    """
    try:
        tag_name, page_obj = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError('pager tag requires 1 argument (page_obj), %s given' % (len(token.split_contents()) - 1))
    return PagerNode(page_obj)

class PagerNode(template.Node):
    def __init__(self, page_obj):
        self.page_obj = template.Variable(page_obj)
    
    def render(self, context):
        page_obj = self.page_obj.resolve(context)
        context = {
            'request': context['request'],
            'page_obj': page_obj,
        }
        return render_to_string('panya/inclusion_tags/pager.html', context)

@register.tag
def render_object(parser, token):
    try:
        tag_name, obj, type = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError('render_object tag requires 2 arguments (obj, type), %s given' % (len(token.split_contents()) - 1))
    return RenderObjectNode(obj, type)

class RenderObjectNode(template.Node):
    def __init__(self, obj, type):
        self.obj = template.Variable(obj)
        self.type = template.Variable(type)

    def render(self, context):
        obj = self.obj.resolve(context)
        type = self.type.resolve(context)

        # update context
        context = copy(context)
        context['object'] = obj

        # generate template name from obj app label, model and type
        obj_type = ContentType.objects.get_for_model(obj)
        template_name = "%s/inclusion_tags/%s_%s.html" % (obj_type.app_label, obj_type.model, type)
        # create response from template. if template is not found for obj type use default content template.
        # if default content template is not found for type return empty response
        try:
            response = render_to_string(template_name, context)
        except TemplateDoesNotExist:
            template_name = "panya/inclusion_tags/modelbase_%s.html" % type
            try:
                response = render_to_string(template_name, context)
            except TemplateDoesNotExist:
                response = ''

        return response

@register.tag
def view_modifier(parser, token):
    """
    Output view modifier.
    """
    try:
        tag_name, view_modifier = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError('view_modifier tag requires 1 argument (view_modifier), %s given' % (len(token.split_contents()) - 1))
    return ViewModifierNode(view_modifier)

class ViewModifierNode(template.Node):
    def __init__(self, view_modifier):
        self.view_modifier = template.Variable(view_modifier)
    
    def render(self, context):
        view_modifier = self.view_modifier.resolve(context)
        context = {
            'request': context['request'],
            'view_modifier': view_modifier,
        }
        return render_to_string('panya/inclusion_tags/view_modifier.html', context)
